@@ -1,7 +1,7 @@
 #!/bin/bash
 
 # SPDX-FileCopyrightText: 2015-2016 Collabora Ltd.
-# SPDX-FileCopyrightText: 2020,2022,2023 Ralf Habacker ralf.habacker @freenet.de
+# SPDX-FileCopyrightText: 2020-2024 Ralf Habacker ralf.habacker @freenet.de
 #
 # SPDX-License-Identifier: MIT
 
@@ -10,17 +10,11 @@ set -x
 
 # kill kde and x session
 function cleanup() {
-    touch finished
-    if test "$ci_in_docker" = yes; then
-        if test "$ci_host" = native; then
-            ${start_kde_session}_shutdown
-            kill -s 9 $DBUS_SESSION_BUS_PID
-        else
-            $wrapper $start_kde_session --terminate
-        fi
-        killall -s 9 xvfb-run
-        sleep 1
-        killall -s 9 Xvfb
+    touch $builddir/finished
+    stop_session
+    stop_webserver
+    if [ "$ci_in_docker" = yes ]; then
+        cleanup_docker
     fi
 }
 
@@ -36,6 +30,12 @@ function start_x_session() {
     sleep 2
 }
 
+function stop_x_session() {
+    killall -s 9 xvfb-run
+    sleep 1
+    killall -s 9 Xvfb
+}
+
 # start dbus-daemon and kde background processes
 function start_kde_session() {
     if test "$ci_host" = native; then
@@ -46,19 +46,85 @@ function start_kde_session() {
     $wrapper $start_kde_session --verbose
 }
 
+function stop_kde_session() {
+    if test "$ci_host" = native; then
+        ${start_kde_session}_shutdown
+        kill -s 9 $DBUS_SESSION_BUS_PID
+    else
+        $wrapper $start_kde_session --terminate
+    fi
+}
+
 function start_session() {
-    if test "$ci_variant" = kf5 && test -v dep_prefix; then
+    if [[ "$ci_variant" =~ "^kf5.*" ]] && test -v dep_prefix; then
         # setup qt5.conf
         qtconf="$dep_prefix/bin/qt5.conf"
         sed "s,Prefix.*$,Prefix=$dep_prefix,g" "$dep_prefix/bin/qt5.conf" > "$builddir/bin/qt5.conf"
     fi
 
+    if [[ "$ci_variant" =~ "^kf6.*" ]] && test -v dep_prefix; then
+        # setup qt6.conf
+        qtconf="$dep_prefix/bin/qt6.conf"
+        sed "s,Prefix.*$,Prefix=$dep_prefix,g" "$dep_prefix/bin/qt6.conf" > "$builddir/bin/qt6.conf"
+    fi
+
     if test "$ci_in_docker" = yes; then
         start_x_session
         start_kde_session
     fi
 }
 
+function stop_session() {
+    if test "$ci_in_docker" = yes; then
+        stop_kde_session
+        stop_x_session
+    fi
+}
+
+# start webserver for testing
+function start_webserver() {
+    if [ "$ci_webserver" = yes ]; then
+        n=$(gawk '$2 ~ TEST_HOST { print $2 }' TEST_HOST=$test_host /etc/hosts)
+        if test -z "$n"; then
+            echo "127.0.0.1 $test_host" | $sudo tee --append /etc/hosts
+        fi
+        $sudo php8 -S "$test_host:80" -t $srcdir/tools &
+    fi
+}
+
+# stop webserver for testing
+function stop_webserver() {
+    if [ "$ci_webserver" = yes ]; then
+        pid=$(ps aux | grep php8 | grep "$test_host" | gawk '{ print $2 }')
+        if test -n "$pid"; then
+            $sudo kill -s 9 $pid
+        fi
+    fi
+}
+
+function init_docker() {
+    mkdir -p ~/.config/gdb
+    cat << EOF > ~/.config/gdb/gdbinit
+print("loading gdb event handler")
+python
+
+def stop_handler (event):
+    print ("event type: stop")
+    gdb.execute("set pagination off")
+    gdb.execute("thread apply all bt")
+    gdb.execute("set confirm off")
+    gdb.execute("quit 1")
+
+gdb.events.stop.connect (stop_handler)
+end
+EOF
+
+}
+
+function cleanup_docker() {
+    rm ~/.config/gdb/gdbinit
+}
+
 # missing wrapper for associated rpm macro
 function cmake-kde4() {
     NAME="cmake_kde4"
@@ -77,6 +143,16 @@ function cmake-kf5() {
     eval "`rpm --eval "%${NAME} $(printf " %q" "${@}")"`"
 }
 
+# missing wrapper for associated rpm macro
+function cmake-kf6() {
+    NAME="cmake_kf6"
+    ${RPM_OPT_FLAGS:=}
+    ${LDFLAGS:=}
+    ${icerun:=}
+    #eval "`rpm --eval "%${NAME} $(printf " %q" "${@}")"`"
+    $(echo "${@}" | sed 's,--,cmake,g')
+}
+
 ##
 ## initialize support to run cross compiled executables
 ##
@@ -179,7 +255,7 @@ init_cross_runtime() {
 
 # ci_in_docker:
 # flags to indicate that we are running in docker
-: "${ci_in_docker:=yes}"
+: "${ci_in_docker:=auto}"
 
 # ci_jobs:
 # number of jobs
@@ -194,36 +270,81 @@ init_cross_runtime() {
 : "${ci_test:=yes}"
 
 # ci_variant:
-# One of kf5, kde4
+# One of kf6, kf5, kf4
 : "${ci_variant:=kf5}"
 
+# ci_webserver:
+# if yes, run simple webserver for testing
+: "${ci_webserver:=yes}"
+
+# host name for testing
+test_host=dev.kmymoney.org
+
 # specify build dir
 srcdir="$(pwd)"
 builddir=${srcdir}/ci-build-${ci_variant}-${ci_host}
 
+# check and setup if running in docker
+# found on https://stackoverflow.com/questions/23513045
+if [ $ci_in_docker = auto ]; then
+    case $(cat /proc/1/sched  | head -n 1 | cut -d' ' -f1) in
+    systemd|init)
+        ci_in_docker=no
+        ;;
+    *)
+        ci_in_docker=yes
+        ;;
+    esac
+fi
 
 # enable sudo if running in docker
 sudo=
-if [ -f /.dockerenv ] && [ -n `getent passwd | grep ^user` ]; then
+if [ "$ci_in_docker" = "yes" ] && [ -n `getent passwd | grep ^user` ]; then
     sudo=sudo
 fi
 
 # common cmake options
 cmake_options="-DBUILD_WITH_QTNETWORK=1"
 
+# check if webserver is enabled
+if [ "$ci_webserver" = yes ]; then
+    # todo add separate host to avoid conflicts host
+    cmake_options+=" -DTEST_DEVELOP_HOST=1"
+fi
+
+# check if running in docker
+if [ "$ci_in_docker" = yes ]; then
+    cmake_options+=" -DTEST_IN_DOCKER=1 -DCMAKE_TEST_LAUNCHER=/usr/bin/gdb;-q;-ex;r;-ex;q"
+    init_docker
+fi
+
+# check if running in gitlab CI
+if [[ -v CI_RUNNER_VERSION ]]; then
+    cmake_options+=" -DTEST_IN_GITLAB_CI=1"
+fi
+
 # settings for build variants
 case "$ci_variant" in
-    (kf5*)
-        cmake_options+=" -DBUILD_APPLETS=0 -DBUILD_TESTING=1 -DENABLE_CLIENT_PACKAGE_TEST=1"
-        cmake_suffix="kf5"
-        export QT_LOGGING_RULES="*=true"
+    (kf6*)
+        cmake_options+=" -DBUILD_APPLETS=0 -DBUILD_TESTING=1 -DENABLE_CLIENT_PACKAGE_TEST=1 -DBUILD_WITH_QT6=1 -DBUILD_WITH_QTNETWORK=1"
+        cmake_suffix="kf6"
+        export QT_LOGGING_RULES="*=true;kf.kio.workers.http.debug=false;qt.text.*.debug=false"
         export QT_FORCE_STDERR_LOGGING=1
         export QT_ASSUME_STDERR_HAS_CONSOLE=1
         start_kde_session=kdeinit5
         ;;
 
-    (kde4)
-        cmake_options+=" -DBUILD_QT4=1 -DKDE4_BUILD_TESTS=1"
+    (kf5*)
+        cmake_options+=" -DBUILD_APPLETS=0 -DBUILD_TESTING=1 -DENABLE_CLIENT_PACKAGE_TEST=1"
+        cmake_suffix="kf5"
+        export QT_LOGGING_RULES="*=true;kf.kio.workers.http.debug=false;qt.text.*.debug=false"
+        export QT_FORCE_STDERR_LOGGING=1
+        export QT_ASSUME_STDERR_HAS_CONSOLE=1
+        start_kde_session=kdeinit5
+        ;;
+
+    (kf4)
+        cmake_options+=" -DBUILD_QT4=1 -DBUILD_TESTING=1"
         cmake_suffix="kde4"
         start_kde_session=kdeinit4
         ;;
@@ -235,20 +356,19 @@ cmake=cmake
 # settings for platforms
 case "$ci_host" in
     (mingw32)
-        # not yet supported
-        ci_test=no
+        cmake_options+=" -DQT_MOC_EXECUTABLE=/usr/i686-w64-mingw32/bin/moc"
         cmake_configure="$ci_host-cmake-$cmake_suffix"
         init_cross_runtime i686-w64-mingw32 $builddir/bin
         wrapper=/usr/bin/wine
         ;;
     (mingw64)
-        # not yet supported
-        ci_test=no
+        cmake_options+=" -DAUTOMOC_EXECUTABLE=/usr/bin/x86_64-w64-mingw32-moc"
         cmake_configure="$ci_host-cmake-$cmake_suffix"
         init_cross_runtime x86_64-w64-mingw32 $builddir/bin
         wrapper=/usr/bin/wine
         ;;
     (*)
+        cmake_options+=" -DCMAKE_CXX_FLAGS=-fPIC"
         cmake_configure="cmake-$cmake_suffix"
         export LD_LIBRARY_PATH=${builddir}/bin
         wrapper=
@@ -256,14 +376,15 @@ case "$ci_host" in
 esac
 
 # custom settings
+cmake_options+=" -DBUILD_WITH_QTNETWORK=1 -DBUILD_WITH_KIO=1"
 case "$ci_variant" in
-    (kf5)
+    (kf[56])
         cmake_options+=" -DBUILD_WITH_WEBKIT=0 -DBUILD_WITH_WEBENGINE=0"
         ;;
-    (kf5-webkit)
+    (kf*-webkit)
         cmake_options+=" -DBUILD_WITH_WEBKIT=1"
         ;;
-    (kf5-webengine)
+    (kf*-webengine)
         cmake_options+=" -DBUILD_WITH_WEBENGINE=1"
         ;;
 esac
@@ -293,8 +414,10 @@ if test "$ci_test" = yes; then
 
     start_session
 
+    start_webserver
+
     # run tests
-    ctest --test-dir ${builddir} --output-on-failure --timeout 60 --jobs $ci_jobs
+    ctest --test-dir ${builddir} --output-on-failure --timeout 60 --parallel $ci_jobs -VV
 
     # show screenshot in case of errors
     if test $? -ne 0; then
